@@ -1,6 +1,6 @@
 /*
  * Copyright 2013 Canonical Ltd.
- * Copyright 2021-2022 Robert Tari
+ * Copyright 2021-2023 Robert Tari
  *
  * Authors:
  *   Charles Kerr <charles.kerr@canonical.com>
@@ -379,30 +379,75 @@ create_admin_section (IndicatorSessionService * self)
 {
   GMenu * menu;
   priv_t * p = self->priv;
-  gchar * desktop_help_label = NULL;
-  gchar * distro_help_label = NULL;
-  if (g_strcmp0(get_desktop_session(), "ubuntu-touch") == 0)
-    {
-      desktop_help_label = g_strdup(_("About Ubuntu Touch…"));
-      distro_help_label  = g_strdup(_("About UBports Foundation…"));
-    }
-    else
-    {
-      desktop_help_label = g_strdup_printf(_("%s Help…"), get_desktop_name());
-      distro_help_label  = g_strdup_printf(_("%s Help…"), get_distro_name());
-    }
   menu = g_menu_new ();
 
-  if (ayatana_common_utils_is_lomiri()) {
-      g_menu_append (menu, _("About This Device…"), "indicator.about");
-  } else {
-      g_menu_append (menu, _("About This Computer"), "indicator.about");
+  gboolean bShowCustom = g_settings_get_boolean (self->priv->indicator_settings, "show-custom-item");
+
+  if (bShowCustom)
+  {
+      gchar *sLabel = g_settings_get_string (self->priv->indicator_settings, "custom-item-label");
+      gchar *sUri = g_settings_get_string (self->priv->indicator_settings, "custom-item-uri");
+
+      if (*sLabel != '\0' && *sUri != '\0')
+      {
+        g_menu_append (menu, sLabel, "indicator.custom");
+      }
+
+      g_free (sUri);
+      g_free (sLabel);
   }
 
-  g_menu_append (menu, desktop_help_label, "indicator.desktop_help");
-  g_menu_append (menu, distro_help_label, "indicator.distro_help");
-  g_free (desktop_help_label);
-  g_free (distro_help_label);
+  gboolean bShowDeviceInfo = g_settings_get_boolean (self->priv->indicator_settings, "show-device-info");
+
+  if (bShowDeviceInfo)
+  {
+      if (ayatana_common_utils_is_lomiri())
+      {
+          g_menu_append (menu, _("About This Device…"), "indicator.about");
+      }
+      else
+      {
+          g_menu_append (menu, _("About This Computer"), "indicator.about");
+      }
+  }
+
+  gboolean bShowDesktopHelp = g_settings_get_boolean (self->priv->indicator_settings, "show-desktop-help");
+
+  if (bShowDesktopHelp)
+  {
+      gchar *desktop_help_label = NULL;
+
+      if (g_strcmp0(get_desktop_session(), "ubuntu-touch") == 0)
+      {
+          desktop_help_label = g_strdup(_("About Ubuntu Touch…"));
+      }
+      else
+      {
+          desktop_help_label = g_strdup_printf(_("%s Help…"), get_desktop_name());
+      }
+
+      g_menu_append (menu, desktop_help_label, "indicator.desktop_help");
+      g_free (desktop_help_label);
+  }
+
+  gboolean bShowDistroHelp = g_settings_get_boolean (self->priv->indicator_settings, "show-distro-help");
+
+  if (bShowDistroHelp)
+  {
+      gchar *distro_help_label = NULL;
+
+      if (g_strcmp0(get_desktop_session(), "ubuntu-touch") == 0)
+      {
+          distro_help_label  = g_strdup(_("About UBports Foundation…"));
+      }
+      else
+      {
+          distro_help_label  = g_strdup_printf(_("%s Help…"), get_distro_name());
+      }
+
+      g_menu_append (menu, distro_help_label, "indicator.distro_help");
+      g_free (distro_help_label);
+  }
 
   if (p->usage_mode_action && ayatana_common_utils_is_lomiri())
   {
@@ -413,7 +458,13 @@ create_admin_section (IndicatorSessionService * self)
       g_object_unref(menu_item);
   }
 
-  g_menu_append (menu, _("Report a Bug…"), "indicator.bug");
+  gboolean bShowBugReport = g_settings_get_boolean (self->priv->indicator_settings, "show-bug-report");
+
+  if (bShowBugReport)
+  {
+      g_menu_append (menu, _("Report a Bug…"), "indicator.bug");
+  }
+
   return G_MENU_MODEL (menu);
 }
 
@@ -1004,6 +1055,13 @@ on_user_activated (GSimpleAction * a         G_GNUC_UNUSED,
                                                 username);
 }
 
+static void on_custom_activated (GSimpleAction *pAction G_GNUC_UNUSED, GVariant *pParam G_GNUC_UNUSED, gpointer pUserData)
+{
+    IndicatorSessionService *self = INDICATOR_SESSION_SERVICE (pUserData);
+    gchar *sUri = g_settings_get_string (self->priv->indicator_settings, "custom-item-uri");
+    ayatana_common_utils_open_url (sUri);
+}
+
 static void
 init_gactions (IndicatorSessionService * self)
 {
@@ -1024,7 +1082,8 @@ init_gactions (IndicatorSessionService * self)
     { "switch-to-screensaver",  on_screensaver_activated     },
     { "switch-to-greeter",      on_greeter_activated         },
     { "suspend",                on_suspend_activated         },
-    { "power-off",              on_power_off_activated       }
+    { "power-off",              on_power_off_activated       },
+    { "custom",                 on_custom_activated          }
   };
 
   p->actions = g_simple_action_group_new ();
@@ -1383,6 +1442,13 @@ indicator_session_service_init (IndicatorSessionService * self)
                             G_CALLBACK(rebuild_header_soon), self);
   g_signal_connect_swapped (gp, "changed::user-show-menu",
                             G_CALLBACK(user_show_menu_changed), self);
+  g_signal_connect_swapped (gp, "changed::show-device-info", G_CALLBACK (rebuild_admin_section_soon), self);
+  g_signal_connect_swapped (gp, "changed::show-desktop-help", G_CALLBACK (rebuild_admin_section_soon), self);
+  g_signal_connect_swapped (gp, "changed::show-distro-help", G_CALLBACK (rebuild_admin_section_soon), self);
+  g_signal_connect_swapped (gp, "changed::show-bug-report", G_CALLBACK (rebuild_admin_section_soon), self);
+  g_signal_connect_swapped (gp, "changed::show-custom-item", G_CALLBACK (rebuild_admin_section_soon), self);
+  g_signal_connect_swapped (gp, "changed::custom-item-label", G_CALLBACK (rebuild_admin_section_soon), self);
+  g_signal_connect_swapped (gp, "changed::custom-item-uri", G_CALLBACK (rebuild_admin_section_soon), self);
 
   /* watch for changes to the lock keybinding */
   gp = p->keybinding_settings;
